/**
 * @BildFilter.java     13.07.2010
 * build: 13.07.2010
 * Copyright 2010 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of SchiebePuzzle
 * Spiele is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * SchiebePuzzle is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Spiele;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.schiebepuzzle;

import java.io.File;

/**
 * 
 * Erweitert EndungsFilter und steuert die Dateiauswahl der im
 * &Ouml;ffnungsdialog anzuzeigenden Dateien und Verzeichnisse
 */
public class BildFilter extends EndungsFilter {

	/**
	 * Zul&auml;ssige Dateiendungen, die von KimA dargestellt werden k&ouml;nnen
	 */
	public static final String[] ALLOWED_EXT = { ".jpg", ".jpeg", ".gif",
			".png" };

	/**
	 * @param f
	 *            File zu pr&uuml;fendes File-Objekt
	 * @return true wenn das File-Objekt ein Verzeichnis ist oder wenn es zum
	 *         erlaubten Dateityp geh&ouml;rt
	 */
	public boolean accept(File f) {
		boolean accept = super.accept(f);
		final String suffix = super.getSuffix(f);

		if (suffix != null) {
			for (int i = 0; i < ALLOWED_EXT.length; i++) {
				if (suffix.toLowerCase().equals(ALLOWED_EXT[i].substring(1))) {
					accept = true;
				}
			}
		}
		return accept;
	}

	/**
	 * Liefert die Dateityp-Beschreibungen der &Ouml;ffnen-Dialoge
	 */
	public String getDescription() {
		String s = "Bild-Dateien ( ";
		for (int i = 0; i < ALLOWED_EXT.length; i++) {
			s += "*." + ALLOWED_EXT[i] + ", ";
		}
		s += " )";
		return s;
	}
}

abstract class EndungsFilter extends javax.swing.filechooser.FileFilter {
	/**
	 * gibt den Endungstyp einer Datei zur&uum;ck
	 * 
	 * @param f
	 *            File-Objekt, dessen Dateityp ermittelt werden soll
	 * @return String Endung
	 */
	public String getSuffix(File f) {
		final String s = f.getPath();
		String suffix = null;
		final int i = s.lastIndexOf('.');

		if ((i > 0) && (i < s.length() - 1)) {
			suffix = s.substring(i + 1).toLowerCase();
		}
		return suffix;
	}

	/**
	 * @param f
	 *            File-Objekt, das daraufhin &uuml;berpr&uuml;ft werden soll ob
	 *            es sich um ein Verzeichnis handelt
	 * 
	 * @return true wenn es ein Verzeichnis ist
	 */
	public boolean accept(File f) {
		return f.isDirectory();
	}
}
