/**
 * @PuzzleModel.java     13.07.2010
 * build: 13.07.2010
 * Copyright 2010 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of SchiebePuzzle
 * Spiele is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * SchiebePuzzle is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Spiele;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.schiebepuzzle;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * Managing image-handling
 * 
 */
public class PuzzleModel {

	private String imgPath;
	private BufferedImage aktBufferedImage;
	private ImageIcon icon;
	private JPanel panel;
	private SchiebePuzzleGUI gui;
	// manages the kind of loading from images true if the first image has to be
	// loaded
	// so, uses a classloader; false if an extern image has to be loaded - uses
	// ImageIO.read(file).
	private boolean isOrigImg;

	public PuzzleModel(SchiebePuzzleGUI gui) {
		this.gui = gui;
		this.panel = gui.getPanel();
	} // Konstruktor

	public void setAktBufferedImage() {
		aktBufferedImage = getScaledImg(loadImage(imgPath), panel);
	}

	public ImageIcon getHelpIcon(JFrame frame) {
		return new ImageIcon(getScaledImg(loadImage(getImgPath()), frame));
	}

	/**
	 * Creates a BufferedImage object from the given path
	 * 
	 * @param imgPath
	 *            String, the path
	 * @return the BufferedImage object
	 */
	public BufferedImage loadImage(String imgPath) {
		BufferedImage buff = null;
		try {
			if (isOrigImg) {
				URL imgName = getClass().getClassLoader().getResource(imgPath);
				buff = ImageIO.read(imgName);
			} else {
				File file = new File(imgPath);
				buff = ImageIO.read(file);
			}

		} catch (IOException e) {
			System.err.println(imgPath + " does not exist!");
			JOptionPane.showMessageDialog(null,
					"Es ist kein Bild vorhanden - Abbruch!");
			System.exit(1);
		} catch (IllegalArgumentException iae) {
			System.err.println(imgPath + " does not exist! Wrong URL?");
			JOptionPane.showMessageDialog(null,
					"Es ist kein Bild mit diesem Pfad vorhanden - Abbruch!");
			System.exit(1);
		}
		return buff;
	}

	/**
	 * Scales the BufferedImage to the size of a given component according to
	 * the images proportions
	 * 
	 * @param bi
	 *            the BufferedImage
	 * @return the scaled BufferedImage
	 */
	BufferedImage getScaledImg(BufferedImage bi, Component comp) {

		int frW = comp.getWidth();
		int frH = comp.getHeight();
		int biW = bi.getWidth();
		int biH = bi.getHeight();
		double faktor = 1;

		if (biW > frW || biH > frH) {
			// wenn beide Seiten des Bildes größer sind als die des Spiels
			double deltaW = (double) biW / (double) frW;
			double deltaH = (double) biH / (double) frH;
			faktor = deltaW > deltaH ? deltaH : deltaW;
		} else {
			// sonst Panelgröße verwenden
			biW = frW;
			biH = frH;
		}

		BufferedImage ergebnis = new BufferedImage((int) (biW / faktor),
				(int) (biH / faktor), BufferedImage.TYPE_INT_ARGB);
		ergebnis.getGraphics().drawImage(bi, 0, 0, (int) (biW / faktor),
				(int) (biH / faktor), null);
		return ergebnis;
	}

	public String getImgPath() {
		return imgPath;
	}

	/**
	 * sets the variables String imgPath and boolean isOrigImg. The default
	 * image is loaded by URL from classpath, an external image is loaded by
	 * file
	 * 
	 * @param path
	 *            String path of the image-file to be set
	 * @param isOrigImg
	 *            boolean determins whether an image is loaded by URL or by file
	 */
	public void setImgPath(String path, boolean isOrigImg) {
		this.isOrigImg = isOrigImg;
		this.imgPath = path;
	}

	/**
	 * Places the image-parts on the elements
	 * 
	 * @param panels
	 */
	public void setImage(SchiebeElement[] panels) {

		panel.setSize(gui.getBorderLength(), gui.getBorderLength());
		panel.setPreferredSize(new Dimension(gui.getBorderLength(), gui.getBorderLength()));
		Image newImg;
		// die Anzahl der Elemente/Kante entspricht der Quadratwurzel aller
		// Elemente
		int div = (int) Math.sqrt(panels.length);

		int frW = panel.getWidth();
		int frH = panel.getHeight();
		// für jedes Element eine Teilbild erstellen und als ImageIcon darauf
		// positionieren
		for (int i = 0; i < div; i++) {
			for (int j = 0; j < div; j++) {
				newImg = aktBufferedImage.getSubimage(j * (frW / div), i
						* (frH / div), frW / div, frH / div);
				icon = new ImageIcon(newImg);
				// leeres Element ausschließen
				if (panels[(i * div + j)] instanceof ImageElement)
					((ImageElement) panels[(i * div + j)]).setImage(icon);
			}
		}
	}
}
