/**
 * @SchiebePuzzleGUI.java     06.07.2010
 * build: 06.07.2010
 * Copyright 2010 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of SchiebePuzzle
 * Test is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * SchiebePuzzle is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Test;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.schiebepuzzle;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.border.LineBorder;

/**
 * Displays a kind of shiftpuzzle based on several square-elements that are
 * arranged as a big square. The play-mode can be changed from pushing the
 * elements for only one square to jumping over several squares. The
 * display-mode can be changed from displaying numbers to displaying a sliced
 * picture.
 */
public class SchiebePuzzleGUI extends JFrame implements MouseListener,
		ActionListener, KeyListener {

	private static final long serialVersionUID = 1L;

	// elements per row/col
	public static int elements = 4;
	private int rows = elements;
	private int cols = elements;

	// colors
	private final Color bgColor = new Color(210, 210, 225);
	private final Color emptyColor = new Color(255, 255, 255);

	// Number of shuffles. The arrays are to be shuffled to place the elements
	// in a random manner.
	// specifies how offen the arrays are shuffled. Each shuffling changes the
	// placees of all the
	// neighbour-elements for the whole array.
	private static final int shuffle = 5;

	// Game mode: true if only the elements next to the empty element can be
	// moved, otherwise false
	private boolean mode = true;

	// Display mode: true if images are to be shown, otherwise show numbers
	private boolean display = true;

	// path of the initial image
	private String imgPath = "img/002.jpg";
	
	// sound file for success notification
	private String soundFile = "de/yourwebs/schiebepuzzle/sound/applauselong.wav";

	// size of the playground-panel
	private final int borderLength = 600;
	private int minCols = 3;
	private int maxCols = 25;

	// font-size
	public static int fontSize = 20;

	// Element-arrays
	private SchiebeElement numberElements[] = new SchiebeElement[rows * cols];
	private SchiebeElement imageElements[] = new SchiebeElement[rows * cols];
	private SchiebeElement emptyElement;
	private Rectangle clickedRectBnds, emptyRectBnds;
	private Rectangle[] origBounds = new Rectangle[rows * cols];

	// GUI-Components
	private JPanel mainPanel;
	private PuzzleModel puzzleModel;
	private JButton newButt, sortButt, closeButt, helpButt, openButt;
	private JTextField field;

	// ToggleButtons with its text
	private JToggleButton modeButt, displayButt;
	private String schiebeModusTxt = "Schieben";
	private String springModusTxt = "Springen";
	private String imgDisplayTxt = "Bild-Puzzle";
	private String numbersDisplayTxt = "Zahlen-Puzzle";

	public SchiebePuzzleGUI() {

		// playground
		mainPanel = new JPanel();
		mainPanel.setLayout(new GridLayout(rows, cols));
		mainPanel.setSize(borderLength, borderLength);
		mainPanel.setPreferredSize(new Dimension(borderLength, borderLength));
		this.setLayout(new BorderLayout());
		this.add(mainPanel, BorderLayout.CENTER);

		// managing images in the model
		puzzleModel = new PuzzleModel(this);
		puzzleModel.setImgPath(imgPath, true);
		puzzleModel.setAktBufferedImage();

		// create all the elements
		createElements();
		if (display) {
			puzzleModel.setImage(imageElements);
			shuffleElements(imageElements);
			arrangeElements(imageElements);
		} else {
			puzzleModel.setImage(numberElements);
			shuffleElements(numberElements);
			arrangeElements(numberElements);
		}

		initButtons();

		this.setTitle("Schiebe-Puzzle");
		this.setIconImage(puzzleModel.loadImage("img/puzzle_512.png"));
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.pack();
		getOrigBounds();
		this.setResizable(false);
		this.setLocationRelativeTo(null);
		this.setVisible(true);
	} // Konstruktor

	private void initButtons() {
		newButt = new JButton();
		newButt.setPreferredSize(new Dimension(22, 22));
		newButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/reload.png")));
		newButt.setToolTipText("Neu mischen");
		newButt.addActionListener(this);

		openButt = new JButton();
		openButt.setPreferredSize(new Dimension(22, 22));
		openButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/fileopen.png")));
		openButt.setToolTipText("Bild laden");
		openButt.addActionListener(this);

		sortButt = new JButton();
		sortButt.setPreferredSize(new Dimension(22, 22));
		sortButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/sort.png")));
		sortButt.setToolTipText("Sortieren");
		sortButt.addActionListener(this);

		closeButt = new JButton();
		closeButt.setPreferredSize(new Dimension(22, 22));
		closeButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/exit.png")));
		closeButt.setToolTipText("Beenden");
		closeButt.addActionListener(this);

		modeButt = new JToggleButton();
		modeButt.setPreferredSize(new Dimension(22, 22));
		modeButt.setSelectedIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/push.png")));
		modeButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/jump.png")));
		modeButt.addActionListener(this);
		setModusText();

		displayButt = new JToggleButton();
		displayButt.setPreferredSize(new Dimension(22, 22));
		displayButt.setSelectedIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/image.png")));
		displayButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/numbers.png")));
		displayButt.addActionListener(this);
		setDisplayButtonText();

		helpButt = new JButton();
		helpButt.setPreferredSize(new Dimension(22, 22));
		helpButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/help.png")));
		helpButt.setToolTipText("Hilfe");
		helpButt.addActionListener(this);

		field = new JTextField(new Integer(elements).toString(), 2);
		field.addKeyListener(this);

		JLabel fieldLabel = new JLabel("Zeilen:");

		JPanel buttPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
		buttPanel.add(fieldLabel);
		buttPanel.add(field);
		buttPanel.add(Box.createHorizontalGlue());
		buttPanel.add(newButt);
		buttPanel.add(openButt);
		buttPanel.add(modeButt);
		buttPanel.add(displayButt);
		buttPanel.add(sortButt);
		buttPanel.add(helpButt);
		buttPanel.add(closeButt);
		this.add(buttPanel, BorderLayout.SOUTH);
	}

	private void rearrange() {
		imageElements = new SchiebeElement[rows * cols];
		numberElements = new SchiebeElement[rows * cols];
		mainPanel.removeAll();
		createElements();
		((GridLayout) mainPanel.getLayout()).setRows(rows);
		((GridLayout) mainPanel.getLayout()).setColumns(cols);
		mainPanel.repaint();
		mainPanel.revalidate();

		if (display) {
			puzzleModel.setImage(imageElements);
			shuffleElements(imageElements);
			arrangeElements(imageElements);
		} else {
			shuffleElements(numberElements);
			arrangeElements(numberElements);
		}

	}

	/**
	 * Creates two arrays of elements: 1. image-elements 2. number-elements the
	 * last element of each SchiebeElement-array is an empty element
	 */
	private void createElements() {
		SchiebeElement sE;
		for (int i = 0; i < rows * cols - 1; i++) {
			// number-elements
			sE = new NumberElement(mainPanel, i + 1);
			sE.setBackground(bgColor);
			sE.addMouseListener(this);
			numberElements[i] = sE;

			// image-elements
			sE = new ImageElement(mainPanel);
			sE.setBackground(bgColor);
			sE.addMouseListener(this);
			imageElements[i] = sE;
		}
		// create an empty element and add it to the array's end
		emptyElement = new SchiebeElement(mainPanel);
		emptyElement.setBackground(emptyColor);
		numberElements[numberElements.length - 1] = emptyElement;
		imageElements[imageElements.length - 1] = emptyElement;
	}

	/**
	 * Selects two elements randomly and switches their position in the
	 * element's array. Not every element must be changed but can be changed
	 * several times
	 */
	private void shuffleElements(SchiebeElement[] sE) {
		int rand1, rand2;
		for (int i = 0; i < sE.length * shuffle; i++) {
			rand1 = (int) (Math.random() * sE.length) + 1;
			rand2 = (int) (Math.random() * sE.length) + 1;

			// switch two elements randomly
			if (rand1 >= sE.length)
				rand1 = 0;
			if (rand1 < 0)
				rand1 = sE.length - 1;
			if (rand2 >= sE.length)
				rand2 = 0;
			if (rand2 < 0)
				rand2 = sE.length - 1;

			// switching
			SchiebeElement temp = sE[rand1];
			sE[rand1] = sE[rand2];
			sE[rand2] = temp;
		}
	}

	/**
	 * Adds all elements of the given array to the panel
	 * 
	 * @param se
	 *            Array to be added
	 */
	private void arrangeElements(SchiebeElement[] se) {
		for (int i = 0; i < se.length; i++) {
			mainPanel.add(se[i]);
		}
	}

	private void getOrigBounds() {
		for (int i = 0; i < origBounds.length; i++) {
			origBounds[i] = mainPanel.getComponent(i).getBounds();
		}
	}

	/**
	 * 
	 * @return the empty element
	 */
	public SchiebeElement getEmptyElement() {
		return emptyElement;
	}

	public int getBorderLength() {
		return borderLength;
	}

	public JPanel getPanel() {
		return mainPanel;
	}

	/**
	 * checks whether an element is a proximat neighbour of the empty element
	 * 
	 * @return true if the asked element is a neighbour otherwise false
	 */
	public boolean isNeighbour() {
		// wenn jedes Element bewegt werden darf, muss
		// der Nachbar nicht ermittelt werden
		if (!mode)
			return true;
		// Position des leeren Rechtecks
		int eRPosX = emptyRectBnds.x;
		int eRPosY = emptyRectBnds.y;
		// Position des angeklickten Rechtecks
		int cRPosX = clickedRectBnds.x;
		int cRPosY = clickedRectBnds.y;
		// Positionen festlegen
		int rectSize = mainPanel.getWidth() / elements;
		boolean o = eRPosX == cRPosX && (eRPosY - rectSize) == cRPosY;
		boolean u = eRPosX == cRPosX && (eRPosY + rectSize) == cRPosY;
		boolean l = (eRPosX - rectSize) == cRPosX && eRPosY == cRPosY;
		boolean r = (eRPosX + rectSize) == cRPosX && eRPosY == cRPosY;
		return o || u || l || r;
	}

	/**
	 * changes the position of an element with that of the empty element by
	 * ascertaining its bounds
	 * 
	 * @param e
	 */
	public void switchElem(MouseEvent e) {
		// only switch neighbours
		if (isNeighbour()) {
			Rectangle tmpBnds = clickedRectBnds;
			clickedRectBnds = emptyRectBnds;
			emptyRectBnds = tmpBnds;

			getEmptyElement().setBounds(emptyRectBnds);
			((SchiebeElement) e.getSource()).setBounds(clickedRectBnds);
		}
	}

	public void verifySuccess() {
		boolean ident = true;
		Rectangle[] tempSE = new Rectangle[numberElements.length];
		for (int i = 0; i < numberElements.length; i++) {
			tempSE[i] = mainPanel.getComponent(i).getBounds();
			if (!(tempSE[i].x == origBounds[i].x)
					|| !(tempSE[i].y == origBounds[i].y))
				ident = false;
		}
		if (ident) {
			playSound();
		}
	}

	private void playSound() {
		URL url = getClass().getClassLoader().getResource(
				soundFile);
		AudioClip sound = null;
		sound = Applet.newAudioClip(url);
		sound.play();
	}

	private void setModusText() {
		String modeButtTxt = mode ? springModusTxt : schiebeModusTxt;
		modeButt.setToolTipText(modeButtTxt);
	}

	private void setDisplayButtonText() {
		String displayButtTxt = display ? numbersDisplayTxt : imgDisplayTxt;
		displayButt.setToolTipText(displayButtTxt);
	}

	/**
	 * sets the path of an external loaded image, registers it at the model, and
	 * display it
	 * 
	 * @param imgPath
	 *            String path to the image
	 */
	public void setExtImg(String imgPath) {
		puzzleModel.setImgPath(imgPath, false);
		puzzleModel.setAktBufferedImage();
		rearrange();
		mainPanel.revalidate();
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
		// getting the bounds of the empty element and its associate partner
		emptyRectBnds = getEmptyElement().getBounds();
		clickedRectBnds = ((SchiebeElement) e.getSource()).getBounds();
		switchElem(e);
		verifySuccess();
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == newButt) {
			if (display) {
				shuffleElements(imageElements);
				arrangeElements(imageElements);
			} else {
				shuffleElements(numberElements);
				arrangeElements(numberElements);
			}
			mainPanel.revalidate();
		}

		// sorting
		if (e.getSource() == sortButt) {
			mainPanel.removeAll();
			createElements();

			if (display) {
				puzzleModel.setImage(imageElements);
				arrangeElements(imageElements);
			} else {
				arrangeElements(numberElements);
			}
			mainPanel.revalidate();
		}

		// Close
		if (e.getSource() == closeButt) {
			System.exit(0);
		}

		// push or jump
		if (e.getSource() == modeButt) {
			mode = !mode;
			setModusText();
		}
		// numbers or image
		if (e.getSource() == displayButt) {
			display = !display;
			setDisplayButtonText();
			mainPanel.removeAll();
			puzzleModel.setImage(imageElements);
			if (display) {
				shuffleElements(imageElements);
				arrangeElements(imageElements);
			} else {
				shuffleElements(numberElements);
				arrangeElements(numberElements);
			}
			mainPanel.revalidate();
		}

		// help
		if (e.getSource() == helpButt) {
			new HelpWindow(puzzleModel);
		}

		// open new image
		if (e.getSource() == openButt) {
			new OpenDialog(this);
		}
	}

	public void keyPressed(KeyEvent e) {
	}

	/**
	 * updating rows/cols by the textfield-entry
	 */
	public void keyReleased(KeyEvent e) {
		if (field.getText().equals(""))
			return;

		for (int i = 0; i < field.getText().length(); i++) {
			if (!Character.isDigit(field.getText().charAt(i))) {
				return;
			}
		}

		int anz = new Integer(field.getText());
		if (e.getSource() == field && anz >= minCols && elements != anz) {
			if (anz > maxCols) {
				JOptionPane.showMessageDialog(this,
						"Maximale Spalten und Reihenzahl: " + maxCols);
				return;
			}
			if (anz > 10)
				fontSize = 15;
			if (anz > 15)
				fontSize = 12;
			if (anz > 20)
				fontSize = 10;
			elements = new Integer(field.getText());
			rows = elements;
			cols = elements;
			rearrange();
		}
	}

	public void keyTyped(KeyEvent e) {
	}
}

class SchiebeElement extends JPanel implements Cloneable {

	private static final long serialVersionUID = 1L;
	JLabel label;

	public SchiebeElement(JPanel frame) {
		int rectSize = frame.getWidth() / SchiebePuzzleGUI.elements;
		this.setPreferredSize(new Dimension(rectSize, rectSize));
		this.setSize(new Dimension(rectSize, rectSize));
		this.setBorder(new LineBorder(Color.WHITE));
		this.setOpaque(true);
		this.setLayout(new GridLayout(1, 1));
		label = new JLabel("", JLabel.CENTER);
		this.add(label);
	} // Konstruktor

	public String getText() {
		return label.getText();
	}

	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return null;
	}
}

class ImageElement extends SchiebeElement {

	private static final long serialVersionUID = 1L;

	public ImageElement(JPanel frame) {
		super(frame);
	}

	public void setImage(ImageIcon icon) {
		label.setIcon(icon);
	}
}

class NumberElement extends SchiebeElement {

	private static final long serialVersionUID = 1L;

	public NumberElement(JPanel frame, int number) {
		super(frame);
		Font font = new Font("Verdana", Font.BOLD, SchiebePuzzleGUI.fontSize);
		label.setFont(font);
		label.setText(new Integer(number).toString());
		label.setSize(this.getWidth(), this.getHeight());
		add(label);
	}
}
